@@ -23,45 +23,54 @@
 SoundConverter Launcher.
 """
 
+
+# imports and package setup
+
 import os
 import sys
 import locale
 import gettext
-from optparse import OptionParser
+from optparse import OptionParser, OptionGroup
 
 # variables
 LIBDIR = '@libdir@'
 DATADIR = '@datadir@'
-
 NAME = 'SoundConverter'
 VERSION = '@version@'
-print(( '%s %s' % (NAME, VERSION) ))
-
 GLADEFILE = '@datadir@/soundconverter/soundconverter.glade'
-
 PACKAGE = NAME.lower()
-try: 
-    locale.setlocale(locale.LC_ALL,'')
-    locale.bindtextdomain(PACKAGE,'@datadir@/locale')
-    gettext.bindtextdomain(PACKAGE,'@datadir@/locale')
+
+try:
+    locale.setlocale(locale.LC_ALL, '')
+    locale.bindtextdomain(PACKAGE, '@datadir@/locale')
+    gettext.bindtextdomain(PACKAGE, '@datadir@/locale')
     gettext.textdomain(PACKAGE)
-    gettext.install(PACKAGE,localedir='@datadir@/locale')
+    gettext.install(PACKAGE, localedir='@datadir@/locale')
     #from gettext import gettext as _
 except locale.Error:
     print('  cannot use system locale.')
-    locale.setlocale(locale.LC_ALL,'C')
+    locale.setlocale(locale.LC_ALL, 'C')
     gettext.textdomain(PACKAGE)
-    gettext.install(PACKAGE,localedir='@datadir@/locale')
+    gettext.install(PACKAGE, localedir='@datadir@/locale')
 
 def _add_soundconverter_path():
-    global localedir
+    """ Makes the soundconverter package importable, which
+    has been installed to LIBDIR during make install """
     root = os.path.join(LIBDIR, 'soundconverter', 'python')
-
     if not root in sys.path:
         sys.path.insert(0, root)
 
+_add_soundconverter_path()
+
+import soundconverter
+soundconverter.NAME = NAME
+soundconverter.VERSION = VERSION
+soundconverter.GLADEFILE = GLADEFILE
+from soundconverter.settings import settings
+from soundconverter.fileoperations import vfs_encode_filename
 
 def _check_libs():
+    """ Tries to import Gst, Gtk and Gdk """
     try:
         import gi
         gi.require_version('Gst', '1.0')
@@ -70,19 +79,26 @@ def _check_libs():
         # force GIL creation - see https://bugzilla.gnome.org/show_bug.cgi?id=710447
         import threading
         threading.Thread(target=lambda: None).start()
-        GLib.threads_init()
+        # GLib.threads_init() # deprecated
         from gi.repository import Gst
         Gst.init(None)
         from gi.repository import Gtk, Gdk
-
     except (ImportError, ValueError) as error:
         print(('%s needs GTK >= 3.0 (Error: "%s")' % (NAME, error)))
         sys.exit(1)
+    # print(( '  using GTK version: %s' % Gtk._version))
+    # print(( '  using Gstreamer version: %s' % (
+    #         '.'.join([str(s) for s in Gst.version()])) ))
 
-    print(( '  using GTK version: %s' % Gtk._version))
-    print(( '  using Gstreamer version: %s' % (
-            '.'.join([str(s) for s in Gst.version()])) ))
+_check_libs()
 
+from soundconverter.batch import CLI_Convert, cli_tags_main, CLI_Check
+from soundconverter.fileoperations import filename_to_uri
+from soundconverter.ui import gui_main
+
+
+
+# command line argument parsing, launch-mode
 
 def check_mime_type(mime):
     types = {'vorbis': 'audio/x-vorbis', 'flac': 'audio/x-flac', 'wav' : 'audio/x-wav',
@@ -91,8 +107,8 @@ def check_mime_type(mime):
     if mime not in list(types.values()):
         print(('Cannot use "%s" mime type.' % mime))
         msg = 'Supported shortcuts and mime types:'
-        for k,v in sorted(types.items()):
-            msg += ' %s %s' % (k,v)
+        for k, v in sorted(types.items()):
+            msg += ' %s %s' % (k, v)
         print(msg)
         raise SystemExit
     return mime
@@ -102,46 +118,83 @@ def mode_callback(option, opt, value, parser, **kwargs):
     setattr(parser.values, option.dest, kwargs[option.dest])
 
 
+class ModifiedOptionParser(OptionParser):
+    """
+    A OptionParser class that doesn't remove newlines on the epilog in order
+    to show usage examples https://stackoverflow.com/questions/1857346/
+
+    See optparse.OptionParser for the original docstring
+    """
+    def format_epilog(self, formatter):
+        if self.epilog == None:
+            return ""
+        return self.epilog
+
+
 def parse_command_line():
-    parser = OptionParser()
+    """ Creates and returns the OptionParser, which parse the
+    command line arguments and displays help with --help. """
+
+    parser = ModifiedOptionParser(epilog='\nExample:\n'
+        '  soundconverter -b [file] [dir] -r -m audio/x-vorbis -s .ogg -o [output dir] -Q 4\n')
+
+    parser.add_option('-c', '--check', dest='mode', action='callback',
+        callback=mode_callback, callback_kwargs={'mode':'check'},
+        help=_('Print which files cannot be read by gstreamer. '
+            'Useful before converting. This will disable the gui and '
+            'run in batch mode.'))
     parser.add_option('-b', '--batch', dest='mode', action='callback',
         callback=mode_callback, callback_kwargs={'mode':'batch'},
         help=_('Convert in batch mode, from command line, '
-            'without a graphical user\n interface. You '
+            'without a graphical user interface. You '
             'can use this from, say, shell scripts.'))
     parser.add_option('-t', '--tags', dest="mode", action='callback',
         callback=mode_callback,  callback_kwargs={'mode':'tags'},
         help=_('Show tags for input files instead of converting '
-            'them. This indicates \n command line batch mode '
+            'them. This indicates command line batch mode '
             'and disables the graphical user interface.'))
-    parser.add_option('-m', '--mime-type', dest="cli-output-type",
-        help=_('Set the output MIME type for batch mode. The default '
-            'is %s. Note that you probably want to set the output '
-            'suffix as well.') % settings['cli-output-type'])
     parser.add_option('-q', '--quiet', action="store_true", dest="quiet",
         help=_("Be quiet. Don't write normal output, only errors."))
     parser.add_option('-d', '--debug', action="store_true", dest="debug",
         help=_('Displays additional debug information'))
-    parser.add_option('-s', '--suffix', dest="cli-output-suffix",
-        help=_('Set the output filename suffix for batch mode.'
-            'The default is %s . Note that the suffix does not '
-            'affect\n the output MIME type.') % settings['cli-output-suffix'])
     parser.add_option('-j', '--jobs', action='store', type='int', dest='forced-jobs',
         metavar='NUM', help=_('Force number of concurrent conversions.'))
-    parser.add_option('--help-gst', action="store_true", dest="_unused",
-        help=_('Shows GStreamer Options'))
+
+    # batch mode settings
+    batch_option_group = OptionGroup(parser, 'Batch Mode Options',
+                        'Those options will only have effect when the -b, -c or -t '
+                        'option is provided')
+    batch_option_group.add_option('-m', '--mime-type', dest="cli-output-type",
+        help=_('Set the output MIME type. The default '
+            'is %s. Note that you probably want to set the output '
+            'suffix as well. Supported shortcuts and mime types: aac '
+            'audio/x-m4a flac audio/x-flac mp3 audio/mpeg vorbis audio/x-vorbis '
+            'wav audio/x-wav') % settings['cli-output-type'])
+    batch_option_group.add_option('-s', '--suffix', dest="cli-output-suffix",
+        help=_('Set the output filename suffix. '
+            'The default is %s. Note that the suffix does not '
+            'affect\n the output MIME type.') % settings['cli-output-suffix'])
+    batch_option_group.add_option('-r', '--recursive', action="store_true", dest="recursive",
+        help=_('Go recursively into subdirectories'))
+    batch_option_group.add_option('-i', '--ignore', action="store_true", dest="ignore-existing",
+        help=_('Ignore files for which the target already exists instead '
+            'of converting them again'))
+    batch_option_group.add_option('-o', '--output', action="store", dest="output-path",
+        help=_('Put converted files into a different directory while rebuilding '
+            'the original directory structure. This includes the name of the original '
+            'directory.'))
+    batch_option_group.add_option('-Q', '--quality', action="store", type='int', dest="quality",
+        metavar='NUM', help=_('Quality of the converted output file. Between 0 '
+            '(lowest) and 5 (highest). Default is 3.'), default=3)
+
+    parser.add_option_group(batch_option_group)
+
+    # not implemented yet
+    # parser.add_option('--help-gst', action="store_true", dest="_unused",
+    #     help=_('Shows GStreamer Options'))
+
     return parser
 
-
-_add_soundconverter_path()
-
-import soundconverter
-soundconverter.NAME = NAME
-soundconverter.VERSION = VERSION
-soundconverter.GLADEFILE = GLADEFILE
-
-from soundconverter.settings import settings
-
 parser = parse_command_line()
 # remove gstreamer arguments so only gstreamer sees them.
 args = [a for a in sys.argv[1:] if '-gst' not in a]
@@ -157,32 +210,24 @@ for k in dir(options):
 
 settings['cli-output-type'] = check_mime_type(settings['cli-output-type'])
 
-_check_libs()
-if settings['forced-jobs']:
-    print(('  using %d thread(s)' % settings['forced-jobs']))
+if not settings.get('quiet'):
+    print(( '%s %s' % (NAME, VERSION) ))
+    if settings['forced-jobs']:
+        print(('Using %d thread(s)' % settings['forced-jobs']))
 
-from soundconverter.batch import cli_convert_main
-from soundconverter.batch import cli_tags_main
-from soundconverter.fileoperations import filename_to_uri
-
-files = list(map(filename_to_uri, files))
-
-try:
-    from soundconverter.ui import gui_main
-except:
-    if settings['mode'] == 'gui':
-        settings['mode'] = 'batch'
+files = [vfs_encode_filename(f) for f in  files]
 
 if settings['mode'] == 'gui':
     gui_main(NAME, VERSION, GLADEFILE, files)
-elif settings['mode'] == 'tags':
-    if not files:
-        print('nothing to do...')
-    cli_tags_main(files)
 else:
     if not files:
         print('nothing to do...')
-    cli_convert_main(files)
+    if settings['mode'] == 'tags':
+        cli_tags_main(files)
+    elif settings['mode'] == 'batch':
+        CLI_Convert(files)
+    elif settings['mode'] == 'check':
+        CLI_Check(files)
 
 
 
